@@ -1,21 +1,69 @@
+# Copyright (c) 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2014 Google, Inc.
+# Copyright (c) 2014 Cole Robinson <crobinso@redhat.com>
+# Copyright (c) 2015-2016, 2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
+# Copyright (c) 2015 David Shea <dshea@redhat.com>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Giuseppe Scrivano <gscrivan@redhat.com>
+# Copyright (c) 2018 Christoph Reiter <reiter.christoph@gmail.com>
+# Copyright (c) 2019 Philipp Hörist <philipp@hoerist.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+
 """Astroid hooks for the Python 2 GObject introspection bindings.
 
 Helps with understanding everything imported from 'gi.repository'
 """
 
+# pylint:disable=import-error,import-outside-toplevel
+
 import inspect
 import itertools
-import sys
 import re
+import sys
 import warnings
 
-from astroid import MANAGER, AstroidBuildingException, nodes
+from astroid import MANAGER, AstroidBuildingError, nodes
 from astroid.builder import AstroidBuilder
 
-
 _inspected_modules = {}
 
-_identifier_re = r'^[A-Za-z_]\w*$'
+_identifier_re = r"^[A-Za-z_]\w*$"
+
+_special_methods = frozenset(
+    {
+        "__lt__",
+        "__le__",
+        "__eq__",
+        "__ne__",
+        "__ge__",
+        "__gt__",
+        "__iter__",
+        "__getitem__",
+        "__setitem__",
+        "__delitem__",
+        "__len__",
+        "__bool__",
+        "__nonzero__",
+        "__next__",
+        "__str__",
+        "__len__",
+        "__contains__",
+        "__enter__",
+        "__exit__",
+        "__repr__",
+        "__getattr__",
+        "__setattr__",
+        "__delattr__",
+        "__del__",
+        "__hash__",
+    }
+)
+
 
 def _gi_build_stub(parent):
     """
@@ -27,7 +75,7 @@ def _gi_build_stub(parent):
     constants = {}
     methods = {}
     for name in dir(parent):
-        if name.startswith("__"):
+        if name.startswith("__") and name not in _special_methods:
             continue
 
         # Check if this is a valid name in python
@@ -36,21 +84,21 @@ def _gi_build_stub(parent):
 
         try:
             obj = getattr(parent, name)
-        except:
+        except AttributeError:
             continue
 
         if inspect.isclass(obj):
             classes[name] = obj
-        elif (inspect.isfunction(obj) or
-              inspect.isbuiltin(obj)):
+        elif inspect.isfunction(obj) or inspect.isbuiltin(obj):
             functions[name] = obj
-        elif (inspect.ismethod(obj) or
-              inspect.ismethoddescriptor(obj)):
+        elif inspect.ismethod(obj) or inspect.ismethoddescriptor(obj):
             methods[name] = obj
-        elif (str(obj).startswith("<flags") or
-              str(obj).startswith("<enum ") or
-              str(obj).startswith("<GType ") or
-              inspect.isdatadescriptor(obj)):
+        elif (
+            str(obj).startswith("<flags")
+            or str(obj).startswith("<enum ")
+            or str(obj).startswith("<GType ")
+            or inspect.isdatadescriptor(obj)
+        ):
             constants[name] = 0
         elif isinstance(obj, (int, str)):
             constants[name] = obj
@@ -64,7 +112,7 @@ def _gi_build_stub(parent):
     ret = ""
 
     if constants:
-        ret += "# %s contants\n\n" % parent.__name__
+        ret += "# %s constants\n\n" % parent.__name__
     for name in sorted(constants):
         if name[0].isdigit():
             # GDK has some busted constant names like
@@ -76,7 +124,7 @@ def _gi_build_stub(parent):
         strval = str(val)
         if isinstance(val, str):
             strval = '"%s"' % str(val).replace("\\", "\\\\")
-        ret += "%s = %s\n" % (name, strval)
+        ret += f"{name} = {strval}\n"
 
     if ret:
         ret += "\n\n"
@@ -98,10 +146,13 @@ def _gi_build_stub(parent):
         ret += "\n\n"
     if classes:
         ret += "# %s classes\n\n" % parent.__name__
-    for name in sorted(classes):
-        ret += "class %s(object):\n" % name
+    for name, obj in sorted(classes.items()):
+        base = "object"
+        if issubclass(obj, Exception):
+            base = "Exception"
+        ret += f"class {name}({base}):\n"
 
-        classret = _gi_build_stub(classes[name])
+        classret = _gi_build_stub(obj)
         if not classret:
             classret = "pass\n"
 
@@ -111,10 +162,11 @@ def _gi_build_stub(parent):
 
     return ret
 
+
 def _import_gi_module(modname):
     # we only consider gi.repository submodules
-    if not modname.startswith('gi.repository.'):
-        raise AstroidBuildingException()
+    if not modname.startswith("gi.repository."):
+        raise AstroidBuildingError(modname=modname)
     # build astroid representation unless we already tried so
     if modname not in _inspected_modules:
         modnames = [modname]
@@ -124,25 +176,30 @@ def _import_gi_module(modname):
         # in pygobject that we need to cope with. However at
         # least as of pygobject3-3.13.91 the _glib module doesn't
         # exist anymore, so if treat these modules as optional.
-        if modname == 'gi.repository.GLib':
-            optional_modnames.append('gi._glib')
-        elif modname == 'gi.repository.GObject':
-            optional_modnames.append('gi._gobject')
+        if modname == "gi.repository.GLib":
+            optional_modnames.append("gi._glib")
+        elif modname == "gi.repository.GObject":
+            optional_modnames.append("gi._gobject")
 
         try:
-            modcode = ''
+            modcode = ""
             for m in itertools.chain(modnames, optional_modnames):
                 try:
-                    __import__(m)
                     with warnings.catch_warnings():
                         # Just inspecting the code can raise gi deprecation
                         # warnings, so ignore them.
                         try:
-                            from gi import PyGIDeprecationWarning
+                            from gi import (  # pylint:disable=import-error
+                                PyGIDeprecationWarning,
+                                PyGIWarning,
+                            )
+
                             warnings.simplefilter("ignore", PyGIDeprecationWarning)
-                        except Exception:
+                            warnings.simplefilter("ignore", PyGIWarning)
+                        except Exception:  # pylint:disable=broad-except
                             pass
 
+                        __import__(m)
                         modcode += _gi_build_stub(sys.modules[m])
                 except ImportError:
                     if m not in optional_modnames:
@@ -155,9 +212,10 @@ def _import_gi_module(modname):
     else:
         astng = _inspected_modules[modname]
     if astng is None:
-        raise AstroidBuildingException('Failed to import module %r' % modname)
+        raise AstroidBuildingError(modname=modname)
     return astng
 
+
 def _looks_like_require_version(node):
     # Return whether this looks like a call to gi.require_version(<name>, <version>)
     # Only accept function calls with two constant arguments
@@ -169,27 +227,32 @@ def _looks_like_require_version(node):
 
     func = node.func
     if isinstance(func, nodes.Attribute):
-        if func.attrname != 'require_version':
+        if func.attrname != "require_version":
             return False
-        if isinstance(func.expr, nodes.Name) and func.expr.name == 'gi':
+        if isinstance(func.expr, nodes.Name) and func.expr.name == "gi":
             return True
 
         return False
 
     if isinstance(func, nodes.Name):
-        return func.name == 'require_version'
+        return func.name == "require_version"
 
     return False
 
+
 def _register_require_version(node):
     # Load the gi.require_version locally
     try:
         import gi
+
         gi.require_version(node.args[0].value, node.args[1].value)
-    except Exception:
+    except Exception:  # pylint:disable=broad-except
         pass
 
     return node
 
+
 MANAGER.register_failed_import_hook(_import_gi_module)
-MANAGER.register_transform(nodes.Call, _register_require_version, _looks_like_require_version)
+MANAGER.register_transform(
+    nodes.Call, _register_require_version, _looks_like_require_version
+)
